// This file is part of mzCalc.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License version 2, as published by the
// Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// mzCalc. If not, see <http://www.gnu.org/licenses/>.
//
// This work is Copyright (C) 2016 by Josse van der Plaat
//
// Individual portions may be copyright by individual contributors, and are
// included in this collective work with permission of the copyright owners.

#ifndef ANALYZESTATS_H
#define ANALYZESTATS_H

struct Stats;

#include "fleet.h"

#include "verbosity.h"

#include <vector>

struct AnalysisOutcome {
    explicit AnalysisOutcome(bool landable = true, bool hasCounter = false, double vulnerability = 0.0)
        : landable(landable)
        , hasCounter(hasCounter)
        , vulnerability(vulnerability)
    { }
    bool landable;
    bool hasCounter;
    double vulnerability;
};
bool operator==(const AnalysisOutcome& lhs, const AnalysisOutcome& rhs);
bool operator!=(const AnalysisOutcome& lhs, const AnalysisOutcome& rhs);

AnalysisOutcome analyzeBattle(const Stats& stats,
                              const Fleet& attacker, const Fleet& defender,
                              const ShipType& attackPod,
                              double valueRatio,
                              bool useRepeatSameTargetIndex,
                              bool enableResourceTheft,
                              bool useFlak,
                              Verbosity verbosity = Verbosity::Warning);

void analyzeStats(const Stats& stats,
                  std::vector<Fleet> fleets,
                  bool useRepeatSameTargetIndex,
                  bool enableResourceTheft,
                  Verbosity verbosity = Verbosity::Warning);

#endif /* ANALYZESTATS_H */
