// This file is part of mzCalc.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License version 2, as published by the
// Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// mzCalc. If not, see <http://www.gnu.org/licenses/>.
//
// This work is Copyright (C) 2016 by Josse van der Plaat
//
// Individual portions may be copyright by individual contributors, and are
// included in this collective work with permission of the copyright owners.

#ifndef BATTLE_H
#define BATTLE_H

#include "fleet.h"
#include "battleformation.h"

#include "verbosity.h"

#include <algorithm>

#include <cstdint>

struct Battle {
    explicit Battle(int64_t attackFleets = 0,
                    int64_t defenseFleets = 0)
    {
        for (int64_t i = 0; i < attackFleets; ++i) {
            fleets.emplace_back("Attack", i, Side::Attack);
        }
        for (int64_t i = 0; i < defenseFleets; ++i) {
            fleets.emplace_back("Defense", i, Side::Defense);
        }
    }
    explicit Battle(const Battle& other)
        : fleets(other.fleets)
    { }
    explicit Battle(Battle&& other)
    {
        std::swap(fleets, other.fleets);
    }
    Battle& operator=(const Battle& other) {
        fleets = other.fleets;
        return *this;
    }
    Battle& operator=(Battle&& other) {
        std::swap(fleets, other.fleets);
        return *this;
    }

    Maybe<const Fleet> fleet(const Side& side, int64_t index) const;
    Maybe<Fleet> fleet(const Side& side, int64_t index);
    Maybe<Fleet> addFleet(const Side& side);

    Maybe<const BattleFormation> battleFormation(const Side& side, int64_t index, const ShipType& type) const;
    Maybe<BattleFormation> battleFormation(const Side& side, int64_t index, const ShipType& type);
    Maybe<const BattleFormation> battleFormation(const Side& side, int64_t index, const char* name) const;
    Maybe<BattleFormation> battleFormation(const Side& side, int64_t index, const char* name);
    bool addBattleFormation(const Side& side, int64_t index, const ShipType& type, NumberOfShips present);

    Cost fleetCost(Side side) const;
    Cost eligibleForSalvage(Side side) const;
    Cost costStolen(Side side) const;
    Cost costLost(Side side) const;
    Cost salvage(Side side) const;
    Cost costChange(Side side) const;

    double vulnerable(Side side, Verbosity verbosity = Verbosity::Warning) const;
    std::string toString() const;

    std::vector<Fleet> fleets;
};
bool operator==(const Battle& lhs, const Battle& rhs);
bool operator!=(const Battle& lhs, const Battle& rhs);

#endif /* BATTLE_H */
