// This file is part of mzCalc.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License version 2, as published by the
// Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// mzCalc. If not, see <http://www.gnu.org/licenses/>.
//
// This work is Copyright (C) 2016 by Josse van der Plaat
//
// Individual portions may be copyright by individual contributors, and are
// included in this collective work with permission of the copyright owners.

#include "shiptype.h"

#include <sstream>
#include <numeric>

std::string Class::toString() const {
    std::string str;
    switch (clazz) {
    case Class::None: {
        str = "None";
    } break;
    case Class::Fighter: {
        str = "Fighter";
    } break;
    case Class::Corvette: {
        str = "Corvette";
    } break;
    case Class::Frigate: {
        str = "Frigate";
    } break;
    case Class::Destroyer: {
        str = "Destroyer";
    } break;
    case Class::Cruiser: {
        str = "Cruiser";
    } break;
    case Class::Battleship: {
        str = "Battleship";
    } break;
    case Class::Asteroid: {
        str = "Asteroid";
    } break;
    case Class::Structure: {
        str = "Structure";
    } break;
    case Class::Resource: {
        str = "Resource";
    } break;
    }
    return str;
}

Class::C Class::fromString(const std::string& str) {
    C clazz = None;
    if (str == "None" || str == "-") {
        clazz = None;
    }
    else if (str == "Fighter" || str == "Fi") {
        clazz = Fighter;
    }
    else if (str == "Corvette" || str == "Co") {
        clazz = Corvette;
    }
    else if (str == "Frigate" || str == "Fr") {
        clazz = Frigate;
    }
    else if (str == "Destroyer" || str == "De") {
        clazz = Destroyer;
    }
    else if (str == "Cruiser" || str == "Cr") {
        clazz = Cruiser;
    }
    else if (str == "Battleship" || str == "Bs") {
        clazz = Battleship;
    }
    else if (str == "Asteroid" || str == "Ro") {
        clazz = Asteroid;
    }
    else if (str == "Structure" || str == "St") {
        clazz = Structure;
    }
    else if (str == "Resource" || str == "Re") {
        clazz = Resource;
    }
    return clazz;
}

bool operator==(const Class& lhs, const Class& rhs) {
    return lhs.clazz == rhs.clazz;
}

bool operator!=(const Class& lhs, const Class& rhs) {
    return !(lhs == rhs);
}

bool operator<(const Class& lhs, const Class& rhs) {
    return lhs.clazz < rhs.clazz;
}

bool operator>=(const Class& lhs, const Class& rhs) {
    return !(lhs < rhs);
}

bool operator<=(const Class& lhs, const Class& rhs) {
    return lhs == rhs || lhs < rhs;
}

bool operator>(const Class& lhs, const Class& rhs) {
    return !(lhs <= rhs);
}

////////////////////////////////////////////////////////////////////////////////

GunType::G GunType::fromString(const std::string& gunTypeName) {
    G result = None;
    if (gunTypeName == "Norm" || gunTypeName == "Kill" || gunTypeName == "Cloak") {
        result = Kill;
    }
    else if (gunTypeName == "Emp" || gunTypeName == "EMP") {
        result = EMP;
    }
    else if (gunTypeName == "Steal") {
        result = Steal;
    }
    else if (gunTypeName == "Pod") {
        result = Pod;
    }
    else if (gunTypeName == "Struc" || gunTypeName == "Structure Kill") {
        result = StructureKill;
    }
    else if (gunTypeName == "Res" || gunTypeName == "Resource Steal") {
        result = ResourceSteal;
    }
    return result;
}

std::string GunType::toString() const {
    std::string result;
    switch (type) {
    case None: {
        result = "None";
    } break;
    case Kill: {
        result = "Kill";
    } break;
    case EMP: {
        result = "EMP";
    } break;
    case Steal: {
        result = "Steal";
    } break;
    case Pod: {
        result = "Pod";
    } break;
    case StructureKill: {
        result = "Structure Kill";
    } break;
    case ResourceSteal: {
        result = "Resource Steal";
    } break;
    }
    return result;
}

bool operator==(const GunType& lhs, const GunType& rhs) {
    return lhs.type == rhs.type;
}

bool operator!=(const GunType& lhs, const GunType& rhs) {
    return !(lhs == rhs);
}

bool operator<(const GunType& lhs, const GunType& rhs) {
    return lhs.type < rhs.type;
}

bool operator>=(const GunType& lhs, const GunType& rhs) {
    return !(lhs < rhs);
}

bool operator<=(const GunType& lhs, const GunType& rhs) {
    return lhs == rhs || lhs < rhs;
}

bool operator>(const GunType& lhs, const GunType& rhs) {
    return !(lhs <= rhs);
}

////////////////////////////////////////////////////////////////////////////////

std::string Race::toString() const {
    std::string result;
    switch (race) {
    case None: { result = "None"; } break;
    case Ter:  { result = "Ter"; } break;
    case Cat:  { result = "Cat"; } break;
    case Xan:  { result = "Xan"; } break;
    case Zik:  { result = "Zik"; } break;
    case Etd:  { result = "Etd"; } break;
    }
    return result;
}

Race::R Race::fromString(const std::string& name) {
    R result = None;
    if (name == "Ter" || name == "Terran") {
        result = Ter;
    }
    else if (name == "Cat" || name == "Cathaar") {
        result = Cat;
    }
    else if (name == "Xan" || name == "Xandathrii") {
        result = Xan;
    }
    else if (name == "Zik" || name == "Zikonian") {
        result = Zik;
    }
    else if (name == "Etd" || name == "Eitraides") {
        result = Etd;
    }
    return result;
}

bool operator==(const Race& lhs, const Race& rhs) {
    return lhs.race == rhs.race;
}

bool operator!=(const Race& lhs, const Race& rhs) {
    return !(lhs == rhs);
}

////////////////////////////////////////////////////////////////////////////////

std::size_t ShipType::targets(Class target) const {
    std::size_t result = MAX_TARGETS;
    std::size_t i = 0;
    while (result == MAX_TARGETS && i < MAX_TARGETS) {
        if (m_targets[i] == target) {
            result = i;
        }
        ++i;
    }
    return result;
}

bool operator==(const ShipType& lhs, const ShipType& rhs) {
    return lhs.name == rhs.name;
}

bool operator!=(const ShipType& lhs, const ShipType& rhs) {
    return !(lhs == rhs);
}

std::string ShipType::toString() const {
    std::stringstream ss;
    ss << name << "(" << clazz.toString() << " -> ";
    ss << std::accumulate(m_targets.cbegin(), m_targets.cend(),
                          std::string(""),
                          [](std::string acc, Class c) {
                              return acc + "," + c.toString();
                          }).substr(1);
    ss << ")";
    return ss.str();
}
