// This file is part of mzLogger.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License version 2, as published by the
// Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// mzLogger. If not, see <http://www.gnu.org/licenses/>.
//
// This work is Copyright (C) 2016 by Josse van der Plaat
//
// Individual portions may be copyright by individual contributors, and are
// included in this collective work with permission of the copyright owners.

#ifndef LOGGER_H
#define LOGGER_H

#include "verbosity.h"

#include <string>

#include <cstdio>

#define implicit

struct Logger {
    implicit Logger(Verbosity verbosity = Verbosity::Warning)
        : verbosity(verbosity)
    { }
    explicit Logger(const std::string& str)
        : verbosity(str)
    { }

    template<typename... T>
    void error(const char* str, T... args) const {
        log(Verbosity::Error,   str, std::forward<T>(args)...);
    }
    template<typename... T>
    void warn(const char* str, T... args) const {
        log(Verbosity::Warning, str, std::forward<T>(args)...);
    }
    template<typename... T>
    void info(const char* str, T... args) const {
        log(Verbosity::Info,   str, std::forward<T>(args)...);
    }
    template<typename... T>
    void debug(const char* str, T... args) const {
        log(Verbosity::Debug,  str, std::forward<T>(args)...);
    }
    template<typename... T>
    void trace(const char* str, T... args) const {
        log(Verbosity::Trace,  str, std::forward<T>(args)...);
    }

    Verbosity verbosity;

private:
    template<typename... T>
    void log(Verbosity verbosity, const char* str, T... args) const {
        if (this->verbosity >= verbosity) {
            std::printf(str, std::forward<T>(args)...);
            std::fflush(stdout);
        }
    }
};

#endif /* LOGGER_H */
