// This file is part of mzTest.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License version 2, as published by the
// Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// mzTest. If not, see <http://www.gnu.org/licenses/>.
//
// This work is Copyright (C) 2016 by Josse van der Plaat
//
// Individual portions may be copyright by individual contributors, and are
// included in this collective work with permission of the copyright owners.

#include <typeinfo>

#include <cstdio>

static int errorCount = 0;

#define EXPECT_EQUAL(actual, expect) expectEqual(__FILE__, __LINE__, __FUNCTION__, actual, expect, #actual, #expect)
template<typename T>
void expectEqual(const char* file, int line, const char* function,
                 const T& actual, const T& expect,
                 const char* actualString, const char* expectString) {
    bool result = actual == expect;
    if (!result) {
        printf("%s:%d: %s expected equal: %s(%s) != %s(%s)\n",
               file,
               line,
               function,
               typeid(actual).name(),
               actualString,
               typeid(expect).name(),
               expectString);
        ++errorCount;
    }
}

#define EXPECT_NONEQUAL(actual, expect) expectNonequal(__FILE__, __LINE__, __FUNCTION__, actual, expect, #actual, #expect)
template<typename T>
void expectInequal(const char* file, int line, const char* function,
                    const T& actual, const T& expect,
                    const char* actualString, const char* expectString) {
    bool result = actual != expect;
    if (!result) {
        printf("%s:%d: %s expected inequal: %s(%s) == %s(%s)\n",
               file,
               line,
               function,
               typeid(actual).name(),
               actualString,
               typeid(expect).name(),
               expectString);
        ++errorCount;
    }
}

#define EXPECT_TRUE(actual) expectTrue(__FILE__, __LINE__, __FUNCTION__, actual, #actual)
template<typename T>
void expectTrue(const char* file, int line, const char* function,
                const T& actual,
                const char* actualString) {
    if (!bool(actual)) {
        printf("%s:%d: %s expected true: %s \n",
               file,
               line,
               function,
               actualString);
        ++errorCount;
    }
}

#define FAIL() fail(__FILE__, __LINE____FUNCTION__, );
void fail(const char* file, int line, const char* function) {
    printf("%s:%d: %s fail\n",
           file,
           line,
           function);
    ++errorCount;
}

void RESET_ERRORS() {
    errorCount = 0;
}

int GET_ERROR_COUNT() {
    return errorCount;
}
